@@ -271,8 +271,9 @@ namespace xt
     template <class T>
     inline auto linspace(T start, T stop, std::size_t num_samples = 50, bool endpoint = true) noexcept
     {
-        double step = double(stop - start) / double(num_samples - (endpoint ? 1 : 0));
-        return cast<T>(detail::make_xgenerator(detail::arange_impl<double>(double(start), double(stop), step), {num_samples}));
+        using fp_type = std::common_type_t<T, double>;
+        fp_type step = fp_type(stop - start) / fp_type(num_samples - (endpoint ? 1 : 0));
+        return cast<T>(detail::make_xgenerator(detail::arange_impl<fp_type>(fp_type(start), fp_type(stop), step), {num_samples}));
     }
 
     /**
@@ -310,7 +311,7 @@ namespace xt
             inline value_type operator()(Args... args) const
             {
                 // TODO: avoid memory allocation
-                return access_impl(xindex({{static_cast<size_type>(args)...}}));
+                return access_impl(xindex({static_cast<size_type>(args)...}));
             }
 
             template <class It>
@@ -324,8 +325,7 @@ namespace xt
 
             inline value_type access_impl(xindex idx) const
             {
-                auto match = [this, &idx](auto& arr)
-                {
+                auto match = [this, &idx](auto& arr) {
                     if (idx[this->m_axis] >= arr.shape()[this->m_axis])
                     {
                         idx[this->m_axis] -= arr.shape()[this->m_axis];
@@ -334,8 +334,7 @@ namespace xt
                     return true;
                 };
 
-                auto get = [&idx](auto& arr)
-                {
+                auto get = [&idx](auto& arr) {
                     return arr[idx];
                 };
 
@@ -371,7 +370,7 @@ namespace xt
             inline value_type operator()(Args... args) const
             {
                 // TODO: avoid memory allocation
-                return access_impl(xindex({{static_cast<size_type>(args)...}}));
+                return access_impl(xindex({static_cast<size_type>(args)...}));
             }
 
             template <class It>
@@ -385,8 +384,7 @@ namespace xt
 
             inline value_type access_impl(xindex idx) const
             {
-                auto get_item = [&idx](auto& arr)
-                {
+                auto get_item = [&idx](auto& arr) {
                     return arr[idx];
                 };
                 size_type i = idx[m_axis];
@@ -524,7 +522,7 @@ namespace xt
         inline auto meshgrid_impl(std::index_sequence<I...>, E&&... e) noexcept
         {
 #if defined X_OLD_CLANG || defined _MSC_VER
-            const std::array<std::size_t, sizeof...(E)> shape { e.shape()[0]... };
+            const std::array<std::size_t, sizeof...(E)> shape = {e.shape()[0]...};
             return std::make_tuple(
                 detail::make_xgenerator(
                     detail::repeat_impl<xclosure_t<E>>(std::forward<E>(e), I),
@@ -535,7 +533,7 @@ namespace xt
             return std::make_tuple(
                 detail::make_xgenerator(
                     detail::repeat_impl<xclosure_t<E>>(std::forward<E>(e), I),
-                    { e.shape()[0]... }
+                    {e.shape()[0]...}
                 )...
             );
 #endif
