@@ -15,8 +15,8 @@
 
 #include "xtl/xsequence.hpp"
 
+#include "xconcepts.hpp"
 #include "xfunction.hpp"
-//#include "xoptional.hpp"
 #include "xscalar.hpp"
 #include "xstrides.hpp"
 
@@ -188,6 +188,12 @@ namespace xt
         template <class Tag, class F, class... E>
         struct select_xfunction_expression;
 
+        template <class F, class... E>
+        struct select_xfunction_expression<xscalar_expression_tag, F, E...>
+        {
+            using type = typename select_xfunction_expression<xtensor_expression_tag, F, E...>::type;
+        };
+
         template <class F, class... E>
         struct select_xfunction_expression<xtensor_expression_tag, F, E...>
         {
@@ -206,6 +212,12 @@ namespace xt
         template <class Tag, template <class...> class F, class... E>
         struct build_functor_type;
 
+        template <template <class...> class F, class... E>
+        struct build_functor_type<xscalar_expression_tag, F, E...>
+        {
+            using type = typename build_functor_type<xtensor_expression_tag, F, E...>::type;
+        };
+
         template <template <class...> class F, class... E>
         struct build_functor_type<xtensor_expression_tag, F, E...>
         {
@@ -221,25 +233,25 @@ namespace xt
         template <class Tag, template <class...> class F, class... E>
         using build_functor_type_t = typename build_functor_type<Tag, F, E...>::type;
 
-        template <template <class...> class F, class... E>
-        inline auto make_xfunction(E&&... e) noexcept
-        {
-            using expression_tag = xexpression_tag_t<E...>;
-            using functor_type = build_functor_type_t<expression_tag, F, E...>;
-            using type = select_xfunction_expression_t<expression_tag, functor_type, const_xclosure_t<E>...>;
-            return type(functor_type(), std::forward<E>(e)...);
-        }
-
         template <template <class...> class F, class... E>
         struct xfunction_type
         {
             using expression_tag = xexpression_tag_t<E...>;
             using functor_type = build_functor_type_t<expression_tag, F, E...>;
             using type = select_xfunction_expression_t<expression_tag,
-                                                       functor_type,
-                                                       const_xclosure_t<E>...>;
+                functor_type,
+                const_xclosure_t<E>...>;
         };
 
+        template <template <class...> class F, class... E>
+        inline auto make_xfunction(E&&... e) noexcept
+        {
+            using function_type = xfunction_type<F, E... >;
+            using functor_type = typename function_type::functor_type;
+            using type = typename function_type::type;
+            return type(functor_type(), std::forward<E>(e)...);
+        }
+
         // On MSVC, the second argument of enable_if_t is always evaluated, even if the condition is false.
         // Wrapping the xfunction type in the xfunction_type metafunction avoids this evaluation when
         // the condition is false, since it leads to a tricky bug preventing from using operator+ and operator-
@@ -590,7 +602,8 @@ namespace xt
      * @return a boolean
      */
     template <class E1, class E2>
-    inline bool operator==(const xexpression<E1>& e1, const xexpression<E2>& e2)
+    inline std::enable_if_t<xoptional_comparable<E1, E2>::value, bool>
+    operator==(const xexpression<E1>& e1, const xexpression<E2>& e2)
     {
         const E1& de1 = e1.derived_cast();
         const E2& de2 = e2.derived_cast();
