@@ -11,7 +11,7 @@
 
 #define XTENSOR_VERSION_MAJOR 0
 #define XTENSOR_VERSION_MINOR 15
-#define XTENSOR_VERSION_PATCH 1
+#define XTENSOR_VERSION_PATCH 2
 
 // DETECT 3.6 <= clang < 3.8 for compiler bug workaround.
 #ifdef __clang__
@@ -28,7 +28,7 @@
 
 #ifndef DEFAULT_SHAPE_CONTAINER
 #define DEFAULT_SHAPE_CONTAINER(T, EA, SA) \
-    std::vector<typename DEFAULT_DATA_CONTAINER(T, EA)::size_type, SA>
+    xt::svector<typename DEFAULT_DATA_CONTAINER(T, EA)::size_type, 4, SA>
 #endif
 
 #ifndef DEFAULT_ALLOCATOR
